#include "fastMatrix.h"

namespace Torch {

fastMatrix::fastMatrix(int n_)
{
   	n = n_;

	//
	transposeMatrix_ = &transposeMatrixnxn_;

	//
	switch(n)
	{
	   case 2:
		initVector_ 		= &initVector2x2_;
		initZeroMatrix_ 	= &initZeroMatrix2x2_;
		initMatrix_ 		= &initMatrix2x2_;
		copyVector_ 		= &copyVector2x2_;
		copyMatrix_ 		= &copyMatrix2x2_;
		vectorialProduct_ 	= &vectorialProduct2x2_;
		product_ 		= &product2x2_;
		addMatrix_ 		= &addMatrix2x2_;
		prodMatrix_ 		= &prodMatrix2x2_;
		divMatrix_ 		= &divMatrix2x2_;
		divVector_ 		= &divVector2x2_;
		normVector_ 		= &normVector2x2_;
		normMatrix_ 		= &normMatrix2x2_;
		inverse_ 		= &inverse2x2_;
		break;

	   case 3:
		initVector_ 		= &initVector3x3_;
		initZeroMatrix_ 	= &initZeroMatrix3x3_;
		initMatrix_ 		= &initMatrix3x3_;
		copyVector_ 		= &copyVector3x3_;
		copyMatrix_ 		= &copyMatrix3x3_;
		vectorialProduct_ 	= &vectorialProduct3x3_;
		product_ 		= &product3x3_;
		addMatrix_ 		= &addMatrix3x3_;
		prodMatrix_ 		= &prodMatrix3x3_;
		divMatrix_ 		= &divMatrix3x3_;
		divVector_ 		= &divVector3x3_;
		normVector_ 		= &normVector3x3_;
		normMatrix_ 		= &normMatrix3x3_;
		inverse_ 		= &inverse3x3_;
		break;

	   case 4:
		initVector_ 		= &initVector4x4_;
		initZeroMatrix_ 	= &initZeroMatrix4x4_;
		initMatrix_ 		= &initMatrix4x4_;
		copyVector_ 		= &copyVector4x4_;
		copyMatrix_ 		= &copyMatrix4x4_;
		vectorialProduct_ 	= &vectorialProductnxn_;
		product_ 		= &productnxn_;
		addMatrix_ 		= &addMatrix4x4_;
		prodMatrix_ 		= &prodMatrix4x4_;
		divMatrix_ 		= &divMatrix4x4_;
		divVector_ 		= &divVector4x4_;
		normVector_ 		= &normVector4x4_;
		normMatrix_ 		= &normMatrix4x4_;
		inverse_ 		= &inversenxn_;
		break;

	   default:
		initVector_ 		= &initVectornxn_;
		initZeroMatrix_ 	= &initZeroMatrixnxn_;
		initMatrix_ 		= &initMatrixnxn_;
		copyVector_ 		= &copyVectornxn_;
		copyMatrix_ 		= &copyMatrixnxn_;
		vectorialProduct_ 	= &vectorialProductnxn_;
		product_ 		= &productnxn_;
		addMatrix_ 		= &addMatrixnxn_;
		prodMatrix_ 		= &prodMatrixnxn_;
		divMatrix_ 		= &divMatrixnxn_;
		divVector_ 		= &divVectornxn_;
		normVector_ 		= &normVectornxn_;
		normMatrix_ 		= &normMatrixnxn_;
		inverse_ 		= &inversenxn_;
		break;
	}

}

void fastMatrix::printVector(real *V)
{
   	print("[ ");
	for(int i = 0 ; i < n ; i++) print("%g ", V[i]);
   	print("]\n");
}
	
void fastMatrix::printMatrix(real **M)
{
	for(int i = 0 ; i < n ; i++)
	{
	   	if(i == 0) print("[ ");
		else print("  ");

		for(int j = 0 ; j < n ; j++) print("%g ", M[i][j]);

		if(i == n-1) print("]\n");
		else print("\n");
	}
}
 
void fastMatrix::initVector(real *V)
{
   	(*initVector_)(n, V);
}

void fastMatrix::initMatrix(real **M)
{
   	(*initZeroMatrix_)(n, M);
}

void fastMatrix::initMatrix(real **M, real d)
{
   	(*initMatrix_)(n, M, d);
}

void fastMatrix::transposeMatrix(real **M)
{
   	(*transposeMatrix_)(n, M);
}

void fastMatrix::copyVector(real *V1, real *V2)
{
   	(*copyVector_)(n, V1, V2);
}

void fastMatrix::copyMatrix(real **M1, real **M2)
{
   	(*copyMatrix_)(n, M1, M2);
}

void fastMatrix::addMatrix(real **M, real **Madd)
{
   	(*addMatrix_)(n, M, Madd);
}

void fastMatrix::prodMatrix(real **M, real d)
{
   	(*prodMatrix_)(n, M, d);
}

void fastMatrix::divMatrix(real **M, real d)
{
   	(*divMatrix_)(n, M, d);
}

void fastMatrix::divVector(real *V, real d)
{
   	(*divVector_)(n, V, d);
}

real fastMatrix::normVector(real *V)
{
   	return (*normVector_)(n, V);
}

real fastMatrix::normMatrix(real **M_)
{
   	return (*normMatrix_)(n, M_);
}

void fastMatrix::vectorialProduct(real **M, real *X, real *T_X)
// Matrix = Vector colonne * Vector ligne
{
   	(*vectorialProduct_)(n, M, X, T_X);
}

void fastMatrix::product(real *VM, real *V, real **M)
// Vector = Vector ligne * Matrix
{
   	(*product_)(n, VM, V, M);
}

real fastMatrix::inverse(real **Minv, real **M1)
{
   	return (*inverse_)(n, Minv, M1);
}

fastMatrix::~fastMatrix()
{
}

//
// 2x2 matrix operations
void initVector2x2_(int n, real *V)
{
	V[0] = 0.0;
	V[1] = 0.0;
}

void initZeroMatrix2x2_(int n, real **M)
{
	M[0][0] = 0.0;
	M[1][0] = 0.0;
	M[0][1] = 0.0;
	M[1][1] = 0.0;
}

void initMatrix2x2_(int n, real **M, real d)
{
	M[0][0] = d;
	M[1][0] = d;
	M[0][1] = d;
	M[1][1] = d;
}

void copyVector2x2_(int n, real *V1, real *V2)
{
	V1[0] = V2[0];
	V1[1] = V2[1];
}

void copyMatrix2x2_(int n, real **M1, real **M2)
{
	M1[0][0] = M2[0][0];
	M1[1][0] = M2[1][0];
	M1[0][1] = M2[0][1];
	M1[1][1] = M2[1][1];
}

void addMatrix2x2_(int n, real **M, real **Madd)
{
	M[0][0] += Madd[0][0];
	M[1][0] += Madd[1][0];
	M[0][1] += Madd[0][1];
	M[1][1] += Madd[1][1];
}

void prodMatrix2x2_(int n, real **M, real d)
{
	M[0][0] *= d;
	M[1][0] *= d;
	M[0][1] *= d;
	M[1][1] *= d;
}

void divMatrix2x2_(int n, real **M, real d)
{
	M[0][0] /= d;
	M[1][0] /= d;
	M[0][1] /= d;
	M[1][1] /= d;
}

void divVector2x2_(int n, real *V, real d)
{
	V[0] /= d;
	V[1] /= d;
}

real normVector2x2_(int n, real *V)
{
	return sqrt(V[0]*V[0] + V[1]*V[1]);
}

real normMatrix2x2_(int n, real **M)
{
	real norm = 0.0;
  
	norm = (M[0][0]*M[0][0]) + (M[1][0]*M[1][0]) + (M[0][1]*M[0][1]) + (M[1][1]*M[1][1]);
      
	return sqrt(norm) / (real) n; 
}

void vectorialProduct2x2_(int n, real **M, real *X, real *T_X)
// Matrix = Vector colonne * Vector ligne
{
	M[0][0] = X[0] * T_X[0];   
	M[0][1] = X[0] * T_X[1];
	M[1][0] = X[1] * T_X[0];   
	M[1][1] = X[1] * T_X[1];   
}

void product2x2_(int n, real *VM, real *V, real **M)
// Vector = Vector ligne * Matrix
{
	VM[0] = V[0] * M[0][0] + V[1] * M[1][0];
	VM[1] = V[0] * M[0][1] + V[1] * M[1][1];
}

real inverse2x2_(int n, real **Minv, real **M1)
{
	real det;

#define A M1[0][0]
#define B M1[0][1]
#define C M1[1][0]
#define D M1[1][1]
	
	det = A*D - B*C;
	
	Minv[0][0] = D / det;
	Minv[0][1] = -B / det;
	Minv[1][0] = -C / det;
	Minv[1][1] = A / det;

#undef A
#undef B
#undef C
#undef D

	return det;
}

//
// 3x3 matrix operations
void initVector3x3_(int n, real *V)
{
	V[0] = 0.0;
	V[1] = 0.0;
	V[2] = 0.0;
}

void initZeroMatrix3x3_(int n, real **M)
{
	M[0][0] = 0.0;
	M[1][0] = 0.0;
	M[2][0] = 0.0;
	M[0][1] = 0.0;
	M[1][1] = 0.0;
	M[2][1] = 0.0;
	M[0][2] = 0.0;
	M[1][2] = 0.0;
	M[2][2] = 0.0;
}

void initMatrix3x3_(int n, real **M, real d)
{
	M[0][0] = d;
	M[1][0] = d;
	M[2][0] = d;
	M[0][1] = d;
	M[1][1] = d;
	M[2][1] = d;
	M[0][2] = d;
	M[1][2] = d;
	M[2][2] = d;
}

void copyVector3x3_(int n, real *V1, real *V2)
{
	V1[0] = V2[0];
	V1[1] = V2[1];
	V1[2] = V2[2];
}

void copyMatrix3x3_(int n, real **M1, real **M2)
{
	M1[0][0] = M2[0][0];
	M1[1][0] = M2[1][0];
	M1[2][0] = M2[2][0];
      
	M1[0][1] = M2[0][1];
	M1[1][1] = M2[1][1];
	M1[2][1] = M2[2][1];
      
	M1[0][2] = M2[0][2];
	M1[1][2] = M2[1][2];
	M1[2][2] = M2[2][2];
}

void addMatrix3x3_(int n, real **M, real **Madd)
{
	M[0][0] += Madd[0][0];
	M[1][0] += Madd[1][0];
	M[2][0] += Madd[2][0];

	M[0][1] += Madd[0][1];
	M[1][1] += Madd[1][1];
	M[2][1] += Madd[2][1];

	M[0][2] += Madd[0][2];
	M[1][2] += Madd[1][2];
	M[2][2] += Madd[2][2];
}

void prodMatrix3x3_(int n, real **M, real d)
{
	M[0][0] *= d;
	M[1][0] *= d;
	M[2][0] *= d;
      
	M[0][1] *= d;
	M[1][1] *= d;
	M[2][1] *= d;
      
	M[0][2] *= d;
	M[1][2] *= d;
	M[2][2] *= d;
}

void divMatrix3x3_(int n, real **M, real d)
{
	M[0][0] /= d;
	M[1][0] /= d;
	M[2][0] /= d;
      
	M[0][1] /= d;
	M[1][1] /= d;
	M[2][1] /= d;
      
	M[0][2] /= d;
	M[1][2] /= d;
	M[2][2] /= d;
}

void divVector3x3_(int n, real *V, real d)
{
	V[0] /= d;
	V[1] /= d;
	V[2] /= d;
}

real normVector3x3_(int n, real *V)
{
	return sqrt(V[0]*V[0] + V[1]*V[1] + V[2]*V[2]);
}

real normMatrix3x3_(int n, real **M)
{
	real norm = 0.0;
	
	norm = 	(M[0][0]*M[0][0]) + (M[1][0]*M[1][0]) + (M[2][0]*M[2][0]) + 
	 	(M[0][1]*M[0][1]) + (M[1][1]*M[1][1]) + (M[2][1]*M[2][1]) +
	 	(M[0][2]*M[0][2]) + (M[1][2]*M[1][2]) + (M[2][2]*M[2][2]);

	return sqrt(norm) / (real) n;
}

void vectorialProduct3x3_(int n, real **M, real *X_, real *T_X)
// Matrix = Vector colonne * Vector ligne
{
      M[0][0] = X_[0] * T_X[0];   
      M[0][1] = X_[0] * T_X[1];   
      M[0][2] = X_[0] * T_X[2];
      
      M[1][0] = X_[1] * T_X[0];   
      M[1][1] = X_[1] * T_X[1];   
      M[1][2] = X_[1] * T_X[2];
      
      M[2][0] = X_[2] * T_X[0];   
      M[2][1] = X_[2] * T_X[1];   
      M[2][2] = X_[2] * T_X[2];   
}

void product3x3_(int n, real *VM, real *V, real **M)
// Vector = Vector ligne * Matrix
{
      VM[0] = V[0] * M[0][0] + V[1] * M[1][0] + V[2] * M[2][0];
      VM[1] = V[0] * M[0][1] + V[1] * M[1][1] + V[2] * M[2][1];
      VM[2] = V[0] * M[0][2] + V[1] * M[1][2] + V[2] * M[2][2];
}

real inverse3x3_(int n, real **Minv, real **M1)
{
	real det;

#define A M1[0][0]
#define B M1[0][1]
#define C M1[0][2]
#define D M1[1][0]
#define E M1[1][1]
#define F M1[1][2]
#define G M1[2][0]
#define H M1[2][1]
#define I M1[2][2]

	det = A * (E*I - H*F) - B * (D*I - G*F) + C * (D*H-G*E);

	Minv[0][0] = (E*I - H*F) / det;
	Minv[0][1] = -(B*I - C*H) / det;
	Minv[0][2] = (B*F - C*E) / det;
	Minv[1][0] = -(D*I - G*F) / det;
	Minv[1][1] = (A*I - C*G) / det;
	Minv[1][2] = -(A*F - C*D) / det;
	Minv[2][0] = (D*H - G*E) / det;
	Minv[2][1] = -(A*H - B*G) / det;
	Minv[2][2] = (A*E - B*D) / det;

#undef A
#undef B
#undef C
#undef D
#undef E
#undef F
#undef G
#undef H

	return det;
}

//
// 4x4 matrix operations
void initVector4x4_(int n, real *V)
{
	V[0] = 0.0;
	V[1] = 0.0;
	V[2] = 0.0;
	V[3] = 0.0;
}

void initZeroMatrix4x4_(int n, real **M)
{
	M[0][0] = 0.0;
	M[1][0] = 0.0;
	M[2][0] = 0.0;
	M[3][0] = 0.0;
	M[0][1] = 0.0;
	M[1][1] = 0.0;
	M[2][1] = 0.0;
	M[3][1] = 0.0;
	M[0][2] = 0.0;
	M[1][2] = 0.0;
	M[2][2] = 0.0;
	M[3][2] = 0.0;
	M[0][3] = 0.0;
	M[1][3] = 0.0;
	M[2][3] = 0.0;
	M[3][3] = 0.0;
}

void initMatrix4x4_(int n, real **M, real d)
{
	M[0][0] = d;
	M[1][0] = d;
	M[2][0] = d;
	M[3][0] = d;
	M[0][1] = d;
	M[1][1] = d;
	M[2][1] = d;
	M[3][1] = d;
	M[0][2] = d;
	M[1][2] = d;
	M[2][2] = d;
	M[3][2] = d;
	M[0][3] = d;
	M[1][3] = d;
	M[2][3] = d;
	M[3][3] = d;
}

void copyVector4x4_(int n, real *V1, real *V2)
{
	V1[0] = V2[0];
	V1[1] = V2[1];
	V1[2] = V2[2];
	V1[3] = V2[3];
}

void copyMatrix4x4_(int n, real **M1, real **M2)
{
	M1[0][0] = M2[0][0];
	M1[1][0] = M2[1][0];
	M1[2][0] = M2[2][0];
	M1[3][0] = M2[3][0];
	M1[0][1] = M2[0][1];
	M1[1][1] = M2[1][1];
	M1[2][1] = M2[2][1];
	M1[3][1] = M2[3][1];
	M1[0][2] = M2[0][2];
	M1[1][2] = M2[1][2];
	M1[2][2] = M2[2][2];
	M1[3][2] = M2[3][2];
	M1[0][3] = M2[0][3];
	M1[1][3] = M2[1][3];
	M1[2][3] = M2[2][3];
	M1[3][3] = M2[3][3];
}

void addMatrix4x4_(int n, real **M, real **Madd)
{
	M[0][0] += Madd[0][0];
	M[1][0] += Madd[1][0];
	M[2][0] += Madd[2][0];
	M[3][0] += Madd[3][0];
	M[0][1] += Madd[0][1];
	M[1][1] += Madd[1][1];
	M[2][1] += Madd[2][1];
	M[3][1] += Madd[3][1];
	M[0][2] += Madd[0][2];
	M[1][2] += Madd[1][2];
	M[2][2] += Madd[2][2];
	M[3][2] += Madd[3][2];
	M[0][3] += Madd[0][3];
	M[1][3] += Madd[1][3];
	M[2][3] += Madd[2][3];
	M[3][3] += Madd[3][3];
}

void prodMatrix4x4_(int n, real **M, real d)
{
	M[0][0] *= d;
	M[1][0] *= d;
	M[2][0] *= d;
	M[3][0] *= d;
	M[0][1] *= d;
	M[1][1] *= d;
	M[2][1] *= d;
	M[3][1] *= d;
	M[0][2] *= d;
	M[1][2] *= d;
	M[2][2] *= d;
	M[3][2] *= d;
	M[0][3] *= d;
	M[1][3] *= d;
	M[2][3] *= d;
	M[3][3] *= d;
}

void divMatrix4x4_(int n, real **M, real d)
{
	M[0][0] /= d;
	M[1][0] /= d;
	M[2][0] /= d;
	M[3][0] /= d;
	M[0][1] /= d;
	M[1][1] /= d;
	M[2][1] /= d;
	M[3][1] /= d;
	M[0][2] /= d;
	M[1][2] /= d;
	M[2][2] /= d;
	M[3][2] /= d;
	M[0][3] /= d;
	M[1][3] /= d;
	M[2][3] /= d;
	M[3][3] /= d;
}

void divVector4x4_(int n, real *V, real d)
{
	V[0] /= d;
	V[1] /= d;
	V[2] /= d;
	V[3] /= d;
}

real normVector4x4_(int n, real *V)
{
	return sqrt(V[0]*V[0] + V[1]*V[1] + V[2]*V[2] + V[3]*V[3]);
}

real normMatrix4x4_(int n, real **M)
{
	real norm = 0.0;
  
	norm = 	(M[0][0]*M[0][0]) + (M[1][0]*M[1][0]) + (M[2][0]*M[2][0]) + (M[3][0]*M[3][0]) +
   		(M[0][1]*M[0][1]) + (M[1][1]*M[1][1]) + (M[2][1]*M[2][1]) + (M[3][1]*M[3][1]) +
   		(M[0][2]*M[0][2]) + (M[1][2]*M[1][2]) + (M[2][2]*M[2][2]) + (M[3][2]*M[3][2]) +
   		(M[0][3]*M[0][3]) + (M[1][3]*M[1][3]) + (M[2][3]*M[2][3]) + (M[3][3]*M[3][3]);

	return sqrt(norm) / (real) n;
}

//
// nxn matrix operations
void initVectornxn_(int n, real *V)
{
	for(int i = 0 ; i < n ; i++) V[i] = 0.0;
}

void initZeroMatrixnxn_(int n, real **M)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++) M[i][j] = 0.0;
}

void transposeMatrixnxn_(int n, real **M)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = i+1 ; j < n ; j++) 
		{
			real tmp_;

			tmp_ = M[i][j];
		   	M[i][j] = M[j][i];
			M[j][i] = tmp_;
		}
}

void initMatrixnxn_(int n, real **M, real d)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++) M[i][j] = d;
}

void copyVectornxn_(int n, real *V1, real *V2)
{
	for(int i = 0 ; i < n ; i++) V1[i] = V2[i];
}

void copyMatrixnxn_(int n, real **M1, real **M2)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++) M1[i][j] = M2[i][j];
}

void addMatrixnxn_(int n, real **M, real **Madd)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++) M[i][j] += Madd[i][j];
}

void prodMatrixnxn_(int n, real **M, real d)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++) M[i][j] *= d;
}

void divMatrixnxn_(int n, real **M, real d)
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++) M[i][j] /= d;
}

void divVectornxn_(int n, real *V, real d)
{
	for(int i = 0 ; i < n ; i++) V[i] /= d;
}

real normVectornxn_(int n, real *V)
{
	real sigma = 0.0;
	
	for(int i = 0 ; i < n ; i++) sigma += V[i]*V[i];
	
	return sqrt(sigma);
}

real normMatrixnxn_(int n, real **M)
{
	real norm = 0.0;
  
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++)
			norm += M[i][j]*M[i][j];

	return sqrt(norm) / (real) n;
}

void vectorialProductnxn_(int n, real **M, real *X, real *T_X)
// Matrix = Vector colonne * Vector ligne
{
	for(int i = 0 ; i < n ; i++)
		for(int j = 0 ; j < n ; j++)
			M[i][j] = X[i] * T_X[j];   
}

void productnxn_(int n, real *VM, real *V, real **M)
// Vector = Vector ligne * Matrix
{
	for(int j = 0 ; j < n ; j++)
		for(int k = 0 ; k < n ; k++)
			VM[j] = VM[j] + V[k] * M[k][j];
}

real inversenxn_(int n, real **Minv, real **M1)
{
   	warning("fast %dx%d matrix inversion not implemented, use matrix package instead.");
	
	return 0.0;
}

}

