#ifndef IPHISTO_INC
#define IPHISTO_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to compute the histogram of pixel values

    \begin{verbatim}

         +---+          +--------------+         +
         |XXX|	        |              |         |   *    *         
	 |XXX|   ---->  |    ipHisto   | ---->   | * *    *  *   *  
         |XXX|          |              |         |** *** *** * * *  
         +---+          +--------------+         +------------------+

	 image                                         histogram
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipHisto : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
public:
	/// normalize the histogram
   	bool normalise;

	//-----
	
	/** constructor

	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	 */
    	ipHisto(int width_, int height_, const char *coding_);

	///
	virtual ~ipHisto();

	///
	void process(Sequence *seq_in);
};

}

#endif
