#include "ipNormMeanStdvLight.h"

namespace Torch {

ipNormMeanStdvLight::ipNormMeanStdvLight(int width_, int height_, const char *coding_, real c_) : ipCore(coding_)
{
	width = width_;
        height = height_;
	
	c = c_;

	seq_out = new(allocator) Sequence(2, width_*height_*n_planes);

	for(int i=0; i<seq_out->frame_size; i++)
        {
                seq_out->frames[0][i] = 0.0;
                seq_out->frames[1][i] = 0.0;
	}							
}

void ipNormMeanStdvLight::process(Sequence *seq_in)
{
	int input_size = width * height;
	
	real *src = seq_in->frames[0];
	real *dst = seq_out->frames[0];
	real *linearlight = seq_out->frames[1];
	real mean_src;
	real stdv_src;
	
	switch(coding_id)
	{
	case 0:
	   	mean_src = 0.0;
	   	stdv_src = 0.0;
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
		
				real z = src[i];
				
				mean_src += z;
				stdv_src += z*z;
			}
		}
		mean_src /= input_size;
		stdv_src /= input_size;
		stdv_src -= mean_src*mean_src;
		stdv_src *= c;
	
		// Normalize 
		if(verbose) 
		{
			print(" + mean src = %g\n", mean_src);
			print(" + stdv src = %g\n", stdv_src);
		}
	
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
				linearlight[i] = (mean_src / stdv_src);
				dst[i] = 128.0 + 255.0 * (src[i] - mean_src) / stdv_src;
			}
		}
		break;
	case 1:
	case 2:
		message("ipNormMeanStdvLight::process() not implemented for RGB and YUV format");
		break;
	case 3:
	   	mean_src = 0.0;
	   	stdv_src = 0.0;
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
		
				real z = src[i];
				
				mean_src += z;
				stdv_src += z*z;
			}
		}
		mean_src /= input_size;
		stdv_src /= input_size;
		stdv_src -= mean_src*mean_src;
		stdv_src *= c;
	
		// Normalize 
		if(verbose) 
		{
			print(" + mean src = %g\n", mean_src);
			print(" + stdv src = %g\n", stdv_src);
		}
	
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
				linearlight[i] = (mean_src / stdv_src);
				dst[i] = 0.5 + (src[i] - mean_src) / stdv_src;
			}
		}
		break;
	}
}

ipNormMeanStdvLight::~ipNormMeanStdvLight()
{
}

}

