#include "ppmDiskXFile.h"

namespace Torch {

ppmDiskXFile::ppmDiskXFile(const char *file_name, const char *open_flags) : ImageDiskXFile(file_name, open_flags)
{
   	n_planes = 3;
	coding = "rgb";
}

bool ppmDiskXFile::readHeader(int *width_, int *height_)
{
	char buffer[200];

   	// Reads Magic Number
	if(!gets(buffer, 200))
	{
		print("ppmDiskXFile::readHeader() not enable to read ppm file\n");
		return false;
	}
	
	if(strcmp(buffer, "P6\n") != 0)
	{
		print("ppmDiskXFile::readHeader() incorrect magic number\n");
		return false;
	}
	
	// Reads comments
	if(!gets(buffer, 200)) 
	{
		print("ppmDiskXFile::readHeader() not enable to read ppm file\n");
		return false;
	}
	//while(buffer[0] == '#') 
	while(buffer[0] == '#' || buffer[0] == '\n') 
		if(!gets(buffer, 200))
		{
			print("ppmDiskXFile::readHeader() not enable to read ppm file\n");
			return false;
		}
	
	//
	// Reads header
   	sscanf(buffer, "%d %d\n", &width, &height);
   	if(width <= 0) 
	{
		print("ppmDiskXFile::readHeader() incorrect width in ppm file\n");
		return false;
	}
   	if(height <= 0)
	{
		print("ppmDiskXFile::readHeader() incorrect height in ppm file\n");
		return false;
	}
	
	//
	// Reads resolution
	if(!gets(buffer, 200))
	{
		print("ppmDiskXFile::readHeader() not enable to read ppm file\n");
		return false;
	}
	
	if(strcmp(buffer, "255\n") != 0)
	{
		print("ppmDiskXFile::readHeader() incorrect resolution, only 255 is supported\n");
		return false;
	}

	if(pixmap != NULL) delete [] pixmap;
	pixmap = new unsigned char [3 * width * height];
	
	*width_ = width;
	*height_ = height;
	
	return true;
}

int ppmDiskXFile::readPixmap()
{
	return read(pixmap, sizeof(unsigned char), 3*width*height);
}

bool ppmDiskXFile::writeHeader(int width_, int height_)
{
   	width = width_;
	height = height;

	printf("P6\n");
	printf("#\n");
	printf("# Image generated using Torch vision\n");
	printf("# (c) 2004-2005 Sebastien Marcel marcel@idiap.ch\n");
	printf("# IDIAP Research Institute\n");
	printf("#\n");
	printf("%d %d\n", width, height);
	printf("255\n");
   
	return true;
}

int ppmDiskXFile::writePixmap(unsigned char *pixmap_)
{
	return write(pixmap_, sizeof(unsigned char), 3*width*height);
}

ppmDiskXFile::~ppmDiskXFile()
{
	if(pixmap != NULL) delete [] pixmap;
}

}
