const char *help = "\
progname: testPoint2D.cc\n\
code2html: This program tests Point2D class methods.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Dan Sunday (http:www.softsurfer.com) and Sebastien Marcel (marcel@idiap.ch)\n";

#include "general.h"
#include "Point2D.h"
#include "Vector2D.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
 	real x1, y1;
 	real x2, y2;
	bool verbose;

	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addRCmdArg("x1", &x1, "x1");
  	cmd.addRCmdArg("y1", &y1, "y1");
  	cmd.addRCmdArg("x2", &x2, "x2");
  	cmd.addRCmdArg("y2", &y2, "y2");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);

	double	 a, c[3];
	Point2D	 P1(1, 0), P2(0,1);
	Point2D	 Q[3], R;
	Vector2D v, w;

	print("sizeof Point2D = %d\n", sizeof(Point2D));
	print("sizeof double = %d\n", sizeof(double));

	// output preset coords of P1 and P2
	print("Initial P1 = %s\n", P1.sprint());
	print("Initial P2 = %s\n", P2.sprint());

	sPoint2D p1 = P1.get();
	print("Initial p1 = (%g, %g)\n", p1.x, p1.y);

	// input new values for Points P1 and P2
	P1.x = x1;
	P1.y = y1;

	P2.x = x2;
	P2.y = y2;

	print("P1 = %s\n", P1.sprint());
	print("P2 = %s\n", P2.sprint());
	
	// test all operators
	v = P1 - P2;		print("P1 - P2 = %s\n", v.sprint());
	R = 2*P1;		print("2*P1 = %s\n", R.sprint());
	R = P1/2;		print("P1/2 = %s\n", R.sprint());
	R = P1 + P2;		print("P1 + P2 = %s\n", R.sprint());
	R = 2*P1 + P2;		print("2*P1 + P2 = %s\n", R.sprint());
	R = (P1 + P2)/2;	print("(P1 + P2)/2 = %s\n", R.sprint());
	R = (P1 + P2)*0.5;	print("(P1 + P2)*0.5 = %s\n", R.sprint());
	print("\n");

	// test all functions
	a = d(P1,P2);		print("d(P1,P2) = %g\n", a);
	a = d2(P1,P2);		print("d2(P1,P2) = %g\n", a);

	c[0]= 0.5; c[1]= 0.5;	print("c[]= {%g, %g}\n", c[0], c[1]);
	Q[0]= P1;  Q[1]= P2;	print("Q[]= {%s, %s}\n", Q[0].sprint(), Q[1].sprint());
	R = asum(2,c,Q);	print("asum(2,c,Q) = %s\n", R.sprint());
}
