const char *help = "\
progname: x2ppm.cc\n\
code2html: This program reads an image and saves it as a ppm.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageRgb.h"
#include "xtprobeImageDiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);

  	ImageDiskXFile *image_file = NULL;
	Image *colorimage = NULL;

	colorimage = new ImageRgb();
	colorimage->setBOption("verbose", verbose);
	
	image_file = new xtprobeImageDiskXFile(image_filename, "r");
	colorimage->loadImageXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", colorimage->width);
		print("   height = %d\n", colorimage->height);
		print("   format = %s (%d)\n", colorimage->coding, colorimage->n_planes);
	}

	colorimage->save("color.ppm");
	
	delete colorimage;

	return(0);
}
