#ifndef HAAR_REAL_STUMP_MACHINE_INC
#define HAAR_REAL_STUMP_MACHINE_INC

#include "RealStumpMachine.h"
#include "ipHaar.h"

namespace Torch {

/** This class is designed to train/support real weak classifiers based on Haar-like features

    @author Yann Rodiguez (rodrig@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class HaarRealStumpMachine : public RealStumpMachine
{   
protected:
	// Size of the pattern where the mask behave
	int width_pattern;
	int height_pattern;

	//
	real *integralimage2;

public:
	/// The current mask to use to compute the feature
	ipHaar *the_mask;
	
	/// All the possible masks (in training mode only)
	ipHaar **masks;
	
	//-----
	
	/// constructor for training
	HaarRealStumpMachine(int n_features_, ipHaar **masks_);	

	/// constructor for testing
	HaarRealStumpMachine(int width_pattern_, int height_pattern_, real *integralimage2_ = NULL);

	/// decode the parameters of the machine
	virtual void setParams(int feature_id_, real min_, real max_, real *lut_);

	/// compute the feature #feature_id_# for the sequence #inputs_# 
        virtual real getFeature(int feature_id_, Sequence *inputs_);

	/// computes the output of the machine given an input sequence
	virtual void forward(Sequence *inputs);

	/// load the parameters of the machine
	virtual void loadXFile(XFile *file);

	/// save the parameters of the machine
	virtual void saveXFile(XFile *file);

	///	
	virtual ~HaarRealStumpMachine();
};

}

#endif
