#include "HaarStumpMachine.h"
#include "XFile.h"

namespace Torch {

HaarStumpMachine::HaarStumpMachine(int n_features_, ipHaar **masks_) : StumpMachine(n_features_)
{
	the_mask = NULL;
	masks = masks_;
}
	
HaarStumpMachine::HaarStumpMachine(int width_pattern_, int height_pattern_, real *integralimage2_)
{
	the_mask = NULL;
	masks = NULL;
	feature_id = 0;
	width_pattern = width_pattern_;
	height_pattern = height_pattern_;
	integralimage2 = integralimage2_;
}

void HaarStumpMachine::setParams(int feature_id_, real threshold_, int direction_)
{
	feature_id = feature_id_;
	threshold = threshold_;
	direction = direction_;

	the_mask = masks[feature_id];
}
       
extern real iisum(real *ii, int width, int height, int x, int y, int w, int h);

real HaarStumpMachine::getFeature(int feature_id_, Sequence *inputs_)
{
   	if(masks != NULL) the_mask = masks[feature_id_];

   	the_mask->process(inputs_);
	
	return the_mask->seq_out->frames[0][0]; 
}

void HaarStumpMachine::loadXFile(XFile *file)
{
	file->taggedRead(&threshold, sizeof(real), 1, "THRESHOLD");
	file->taggedRead(&direction, sizeof(int), 1, "DIR");
	int mask_id_, x_, y_, w_, h_;
        file->taggedRead(&mask_id_, sizeof(int), 1, "MASK_ID");
        file->taggedRead(&x_, sizeof(int), 1, "X_MASK");
        file->taggedRead(&y_, sizeof(int), 1, "Y_MASK");
        file->taggedRead(&w_, sizeof(int), 1, "W_MASK");
        file->taggedRead(&h_, sizeof(int), 1, "H_MASK");
	
	switch(mask_id_)
        { 
        case 1: 
                 the_mask = new(allocator) biVertHaar(width_pattern, height_pattern, x_, y_, w_, h_); /* Bi-Vertical */
                 break;
        case 2:
                 the_mask = new(allocator) biHorizHaar(width_pattern, height_pattern, x_, y_, w_, h_); /* Bi-Horizontal */
                 break;
        case 3:
                 the_mask = new(allocator) diagHaar(width_pattern, height_pattern, x_, y_, w_, h_); /* Diagonal */
                 break;
        case 4:
                 the_mask = new(allocator) triVertHaar(width_pattern, height_pattern, x_, y_, w_, h_); /* Tri-Vertical */
                 break;
        case 5:
                 the_mask = new(allocator) triHorizHaar(width_pattern, height_pattern, x_, y_, w_, h_); /* Tri-Horizontal */
                 break;
	case 6:
		 the_mask = new(allocator) centralHaar(width_pattern, height_pattern, x_, y_, w_, h_); /* Central */
		 break;
        }

	if(the_mask == NULL)
		error("HaarStumpMachine::loadXFile() impossible to allocate a Haar mask");

	the_mask->integralimage2 = integralimage2;
}

void HaarStumpMachine::saveXFile(XFile *file)
{
	file->taggedWrite(&threshold, sizeof(real), 1, "THRESHOLD");
	file->taggedWrite(&direction, sizeof(int), 1, "DIR");
	the_mask->saveXFile(file);
}

HaarStumpMachine::~HaarStumpMachine()
{
}

}
