#ifndef MAHANALOBIS_INC
#define MAHANALOBIS_INC

#include "Metric.h"
#include "EigenMachine.h"

namespace Torch {

/** This class is designed to handle the Mahanalobis distance

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mMahanalobis : public Metric
{
protected:
	/// dimension of the projection
	int n_outputs;

   	/// projection of #sample_# vector into Eigen subspace
   	real *m;

   	/// projection of #template_# vector into Eigen subspace
	real *n;

	/// standard deviation
	real *stdv;

	/// input sequence for the EigenMachine
	Sequence *seq;
	
public:
   	/// the eigen machine (PCA for instance)
   	EigenMachine *eigen_machine;
	
	//-----
	
	///
	mMahanalobis(int n_inputs_, EigenMachine *eigen_machine_);

	///
	void projection(real *sample_, real *template_);

	///
	virtual ~mMahanalobis() {};
};

/** This class is designed to compute the Mahanalobis L1 distance

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mMahanalobisL1 : public mMahanalobis
{
public:
	///
	mMahanalobisL1(int n_inputs_, EigenMachine *eigen_machine_);

	///
	virtual real measure(real *sample_, real *template_ = NULL);

	///
	virtual ~mMahanalobisL1() {};
};

/** This class is designed to compute the Mahanalobis L2 distance

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mMahanalobisL2 : public mMahanalobis
{
public:
	///
	mMahanalobisL2(int n_inputs_, EigenMachine *eigen_machine_);

	///
	virtual real measure(real *sample_, real *template_ = NULL);

	///
	virtual ~mMahanalobisL2() {};
};

/** This class is designed to compute the Mahanalobis Cosine distance

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mMahanalobisCosine : public mMahanalobis
{
public:
	///
	mMahanalobisCosine(int n_inputs_, EigenMachine *eigen_machine_);

	///
	virtual real measure(real *sample_, real *template_ = NULL);

	///
	virtual ~mMahanalobisCosine() {};
};

}

#endif

