#ifndef TANGENT_INC
#define TANGENT_INC

#include "Metric.h"

namespace Torch {

/** This class is designed to compute the Tangent distance

    It uses tangent distance routines from Daniel Keysers 
    available \URL[here]{http://www-i6.informatik.rwth-aachen.de/~keysers/td}.
    
    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 2.0
*/
class mTangentDistance : public Metric
{
	// definitions for the implicit smoothing template
	#define templatefactor1 0.1667
	#define templatefactor2 0.6667
	#define templatefactor3 0.08
	/* the template looks like this (extended Sobel):
	   0   f1 0 -f1  0
	   f3  f2 0 -f2 -f3
	   0   f1 0 -f1  0  */
	/* note: this template works well for 16x16 sized images, 
	   for different sizes an additional smoothing might be helpful */


	/// constant for brightness tangent
	double additiveBrightnessValue;

	/// constant size of "choice" equal the max number of tangents
	int maxNumTangents;
	int *choice;

	//
	double background;

	/// calculates the distance between two images given a set of orthonormalized tangents
	real calculateDistance(real* imageOne, real* imageTwo, double** tangents, int numTangents);
	   
	/** calculates an orthonormal basis using Gram-Schmidt
	
	    returns 0 
	    sets tangents to zero, that are not "orthogonal enough" 
	*/
	int orthonormalizePzero(double **A, int num, int dim);

	/** Finds an orthonormal basis for a given set of tangents

	  	@param tangents The tangents to be orthonormalized.
		@param numTangents The number of tangents to be found.
		@param height The height of the tangents.
		@param width The width of the tangents.
	*/
	int normalizeTangents(double ** tangents, int numTangents);

	//
	int calculateTangents(real* image, double ** tangents, int numTangents);

	//
	real tangentDistance(real* imageOne, real* imageTwo);
	
public:
   	/// width of the input image
	int width;
	
	/// height of the input image
	int height;
	
	//-----
	
	///
	mTangentDistance(int width_, int height_);

	///
	virtual real measure(real *sample_, real *template_ = NULL);

	///
	virtual ~mTangentDistance() {};
};

}

#endif

