#ifndef IP_SUBWINDOW_MLP_INC
#define IP_SUBWINDOW_MLP_INC

#include "ipSubWindow.h"

#include "ImageGray.h"

#include "ipHistoEqual.h"
#include "ipSmoothGaussian3.h"

#include "MyMLP.h"
#include "MyMeanVarNorm.h"

namespace Torch {

/** This class is designed to classify a subwindow using a MLP
 
    This class should be used with a PyramiWindowScan.
	
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class ipSWmlp : public ipSubWindow
{
	// gray image for verbosing purpose
	ImageGray *grayimage;

public:
	/// pattern size
        int size_pattern;

	/// the MLP
	MyMLP *mlp;

	/// the normalization
  	MyMeanVarNorm *mv_norm;

	/// the image normalization preprocessing
	ipCore *enhancing;

	/// the smoothing
	ipCore *smoothing;

	/// the sub_image to store the current input image to process
	Sequence *sub_image;

	//-----
	
	///
	ipSWmlp(int width_image_, int height_image_, 
		int width_pattern_, int height_pattern_, 
		const char *model_filename_, real threshold_,
		bool verbose_ = false);

	///
	virtual void process(Sequence *seq_in);

	///
	virtual ~ipSWmlp();
};

}

#endif
