#ifndef BOOSTING_REAL_INC
#define BOOSTING_REAL_INC

#include "Trainer.h"
#include "WeightedSumMachine.h"
#include "ClassFormat.h"

namespace Torch {

/** Real Boosting implementation

    As the idea of boosting in regression hasn't been really well tested,
    this is boosting for *classification* only.
    
    This trainer will "boost" the machine given by the #WeightedSumMachine#,
    on the given #DataSet#.

    You have to provide a #ClassFormat# to know how the classes are encoded.

    @author Yann Rodriguez (rodrig@idiap.ch)
    @author Ronan Collobert (collober@idiap.ch)

    @version 2.0
    \Date
    @since 2.0

    @see WeightedSumMachine
*/
class BoostingReal : public Trainer
{
public:

	/// This machine performs the combination. It contains many trainers.
	WeightedSumMachine *w_machine;

	/// #ClassFormat# to know how the classes are encoded.
	ClassFormat *class_format;

	/// The number of trainers in the boosting.
	int n_trainers;

	/// The weights of each machine in the boosting.
	real *weights;

	///
	BoostingReal(WeightedSumMachine *w_machine_, ClassFormat *class_format_);

	//-----
    
	///
	virtual void train(DataSet *data, MeasurerList *measurers);

	///
	virtual ~BoostingReal();
};


}

#endif
