#ifndef DISK_BIN_DATA_SET_INC
#define DISK_BIN_DATA_SET_INC

#include "DiskDataSet.h"

namespace Torch {

/** This class is designed to manipulate binary files from disk (data NOT fully loaded in memory)

    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)

    @version 2.0
    \Date
    @since 1.0
*/
class DiskBinDataSet : public DiskDataSet
{
public:
	/** @name Lots of constrcutors */
        //@{
	/** load examples from a binary file and assign a target
	
	    @param filename_ is the file to read
	    @param n_inputs_ is the size of the examples
	    @param target_ is the value of the target to assign to each example
	    @param max_load is the maximum number of examples to load
	*/
	DiskBinDataSet(const char *filename_, int n_inputs_, real target_, int max_load);

	/** load examples from a binary file (NO targets assigned)
	
	    @param filename_ is the file to read
	    @param n_inputs_ is the size of the examples
	    @param max_load is the maximum number of examples to load
	*/
	DiskBinDataSet(const char *filename_, int n_inputs_, int max_load); 
    
	/** load examples from a multiple files and assign the same target to all the examples from the same file
	
	    @param filenames_ is the array of files to read
	    @param n_files_ is the number of files in the array
	    @param n_inputs_ is the size of the examples
	    @param targets_ is the array of targets to assign
	    @param max_load is the maximum number of examples to load
	*/
	DiskBinDataSet(char **filenames_, int n_files_, int n_inputs_, real *targets_, int max_load);
    
	/** load examples from a multiple files (NO targets assigned)
	
	    @param filenames_ is the array of files to read
	    @param n_files_ is the number of files in the array
	    @param n_inputs_ is the size of the examples
	    @param max_load is the maximum number of examples to load
	*/
	DiskBinDataSet(char **filenames_, int n_files_, int n_inputs_, int max_load);
    
	/** load examples from a multiple files and assign the same target to all the examples
	
	    @param filenames_ is the array of files to read
	    @param n_files_ is the number of files in the array
	    @param n_inputs_ is the size of the examples
	    @param target_ is the value of the target to assign
	    @param max_load is the maximum number of examples to load
	*/
	DiskBinDataSet(char **filenames_, int n_files_, int n_inputs_, real target_, int max_load);
   	//@}
	
	// Print only the header if val=false (default); the full data if val=true
	void info(bool val=false);

	// save the dataset in the file #filename_# (binary)
	void save(const char *filename_);

	//
	virtual ~DiskBinDataSet();
};

}

#endif
