#ifndef FILE_BIN_DATA_SET_INC
#define FILE_BIN_DATA_SET_INC

#include "MemoryDataSet.h"

namespace Torch {

/** This class is designed to load bindata files and add them targets if needed (data fully loaded in memory)

    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @author Fabien Cardinaux (cardinaux@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)

    @version 2.0
    \Date
    @since 1.0

    @see IOBin
*/
class FileBinDataSet : public MemoryDataSet
{
 public:
    /** load a file.
        \begin{itemize}
          \item #filename# is the name of the file to load (yes..).
          \item #n_inputs_# is the dim of the input.
          \item #target# is the target (real value) assigned to each pattern of the #filename# file.
          \item #max_load_# is the number of patterns in the file to load.
        \end{itemize}
    */   
    FileBinDataSet(char *filename, int n_inputs_, real target, int max_load = -1, bool verbose_ = false);

    /** exactly the same, but without assigning a target
    */
    FileBinDataSet(char *filename, int n_inputs_, int max_load = -1, bool verbose_ = false); 
    
    /** load a set of files.
        \begin{itemize}
          \item #filenames# is the name of the files...
          \item #n_files# is the number of files to load.
          \item #n_inputs_# is the dim of the input (same for all files!).
          \item #target# is the target (real value) assigned to each pattern of the #filename# file.
          \item #max_load_# is the number of patterns in the file to load.
        \end{itemize}
    */   
    FileBinDataSet(char **filenames, int n_files, int n_inputs_, real *targets, int max_load = -1, bool verbose_ = false);
    FileBinDataSet(char **filenames, int n_files, int n_inputs_, int offset_window, int n_inputs_window, real *targets, int max_load = -1, bool verbose_ = false);
    
    /** load a set of files of 2 class.
    */   
    FileBinDataSet(char **filenames_class1, int n_files_class1, real target_class1, 
	  	   char **filenames_class2, int n_files_class2, real target_class2, 
	  	   int n_inputs_, bool verbose_ = false);
   
    /** load a set of files
    */
    FileBinDataSet(char **filenames_class, int n_files_class, real target_class, int n_inputs_, bool verbose_ = false);
    FileBinDataSet(char **filenames_class, int n_files_class, real target_class, int n_inputs_, int offset_window, int n_inputs_window, bool verbose_ = false);
    FileBinDataSet(char **filenames_class, int n_files_class, int n_inputs_, int offset_window, int n_inputs_window, bool verbose_ = false);
    FileBinDataSet(char **filenames_class, int n_files_class, int n_inputs_, bool verbose_ = false);
    
    /** Print only the header if display_=false (default);
	the full data if display_=true.
    */ 
    void info(bool display_=false);

    virtual ~FileBinDataSet();
};

}

#endif
