#ifndef PCA_TRAINER_INC
#define PCA_TRAINER_INC

#include "Trainer.h"
#include "EigenTrainer.h"
#include "PCAMachine.h"

namespace Torch {

/** This class is designed to compute Principal Component Analysis

    @author Sebastien Marcel (marcel@idiap.ch)
    \Date
    @version 2.0
    @since 1.0
*/
class PCATrainer : public Trainer
{
public:
   	/// number of inputs
	int n_inputs;

   	/// covariance matrix of the data
	Mat *CovMat;
   	
	/// mean vector
	real *Xm_;

	/// eigenvectors
    	Mat *eigenvectors;

	/// eigenvectors of the PCAMachine (to be filled)
	real **eigenvectors_;

	/// eigenvalues
    	Vec *eigenvalues;

	/// eigenvalues of the PCAMachine (to be filled)
	real *eigenvalues_;

	/// verbosity level
	int verbose_level;

	/// save the covariance matrix
	bool saveCovar;
	
	//-----
	
	///
	PCATrainer(PCAMachine *machine_);

	///
	virtual void train(DataSet *data_, MeasurerList *measurers_ = NULL);

	///
	virtual ~PCATrainer();
};

}

#endif
