#ifndef TIMECODE_INC
#define TIMECODE_INC

#include "general.h"

namespace Torch {
   
/** This class is designed to handle a timecode

    A timecode is a hh:mm:ss:ff timestamp.
	
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class TimeCode
{
protected:
   	/// the timecode
	int timecode[4];

public:
	/** @name constructors */
	//@{
	///
	TimeCode();

	///
	TimeCode(int h_, int m_, int s_, int f_);
	//@}
	
	/// destructor
	virtual ~TimeCode();
	
	//-----
	
	/// return the hour timestamp
	int hh() { return timecode[0]; };

	/// return the minute timestamp
	int mm() { return timecode[1]; };

	/// return the second timestamp
	int ss() { return timecode[2]; };

	/// return the frame timestamp
	int ff() { return timecode[3]; };

	//-----
	
	/// reset the timecode
	void reset(int h_, int m_, int s_, int f_);
	
	/// reset the timecode from an image
	virtual bool reset(unsigned char *pixmap_) = 0;
	
};

}

#endif
