#include "VideoFile.h"

namespace Torch {

VideoFile::VideoFile()
{
	frame = NULL;
	pixmap = NULL;
	cframe = 0;

	filename = NULL;
	open_flags = NULL;
}
  	
VideoFile::VideoFile(char *filename_, const char *open_flags_)
{
	frame = NULL;
	pixmap = NULL;
	cframe = 0;

	filename = filename_;
	open_flags = (char *) open_flags_;
}
  	
bool VideoFile::initVideoSource()
{
	if(filename == NULL) status = VIDEOSOURCE_ERROR;

  	if(status == VIDEOSOURCE_ERROR) return false;  

	return true; 
}

bool VideoFile::openVideoSource()
{
  	if(status == VIDEOSOURCE_OK)
	{
		status = VIDEOSOURCE_OPEN;
		
		//print("VideoFile::open() opening file %s\n", filename);
		
		if(open(filename, open_flags) == false) return false;
	
		if (strcmp(open_flags, "w")) 
		{
			print("   width = %d\n", width);
			print("   height = %d\n", height);
			print("   nframes = %d\n", nframes);
			print("   fps = %g\n", frate);
		}
		
		if(frame != NULL) delete frame;
		frame = new FrameVideoFile(width, height);
		if(frame == NULL)
		{
			status = VIDEOSOURCE_ERROR;

			return false;
		}

     	 	return true;
	}
  	else
	{
		warning("VideoFile: Impossible to open.");

      		return false;
	}
	
}

bool VideoFile::closeVideoSource()
{
  	if(status == VIDEOSOURCE_OPEN) return close();

	return false; 
}

VirtualPixmap * VideoFile::getframe()
{
  	if(status != VIDEOSOURCE_OPEN)
	{
		warning("VideoFile: Impossible to get a frame.");

      		return NULL;
	}
  
	if(read() == true) frame->copyBuffer(pixmap);
	else return NULL;

  	return frame;
}

VideoFile::~VideoFile()
{
	if(frame != NULL) delete frame;
}

}

