#include "ffmpegVideoFile.h"
#include "Convert.h"

namespace Torch {

int ffmpegVideoFile::n_instance = 0;

ffmpegVideoFile::ffmpegVideoFile()
{
#ifndef HAVE_FFMPEG
	error("HAVE_FFMPEG not defined in ffmpegVideoFile::ffmpegVideoFile()");
#endif
   
	readMode = false;
	writeMode = false;

	addIOption("width", &width, 720, "output width");
	addIOption("height", &height, 576, "output height");
	addIOption("bitrate", &bitrate, 1500000, "output bitrate");
	addROption("framerate", &frate, 25, "output framerate");
	addIOption("gop", &gop, 100, "output gop size");

#ifdef HAVE_FFMPEG
	ic = NULL;
	oc = NULL;
	iformat = NULL;
	st = NULL;
	pkt = NULL;
	c = NULL;
	codec = NULL;
	avframe = NULL;
	pict = NULL;
	pb = NULL;
	outbuf = NULL;

	n_instance++;

	if (n_instance==1) 
	{
		avcodec_init();
		av_register_all();
	}
#endif
}
	
ffmpegVideoFile::ffmpegVideoFile(char *filename_, const char *open_flags_) : VideoFile(filename_, open_flags_)
{
#ifndef HAVE_FFMPEG
	error("HAVE_FFMPEG not defined in ffmpegVideoFile::ffmpegVideoFile()");
#endif
   
	readMode = false;
	writeMode = false;

	addIOption("width", &width, 720, "output width");
	addIOption("height", &height, 576, "output height");
	addIOption("bitrate", &bitrate, 1500000, "output bitrate");
	addROption("framerate", &frate, 25, "output framerate");
	addIOption("gop", &gop, 100, "output gop size");

#ifdef HAVE_FFMPEG
	ic = NULL;
	oc = NULL;
	iformat = NULL;
	st = NULL;
	pkt = NULL;
	c = NULL;
	codec = NULL;
	avframe = NULL;
	pict = NULL;
	pb = NULL;
	outbuf = NULL;

	n_instance++;

	if (n_instance==1) 
	{
		avcodec_init();
		av_register_all();
	}
#endif
}
  	
bool ffmpegVideoFile::open(char *filename_, const char *open_flags_)
{
	readMode = false;
	writeMode = false;

	cframe = 0;

#ifdef HAVE_FFMPEG
	if(ic != NULL) 
	{
		print("You already do open...");
		return false;
	}

	// Do not show warning/errors while decoding
	// -1: show nothing
	// 0 : show warning
	// 1 : shaw warning + errors
	// 2 : debugg mode
	av_log_set_level(-1);

	if(strcmp(open_flags_, "r") == 0) 
	{
		//--------------------------------------------------------------------------
		//
		//		R E A D       I N I T I A L I S A T I O N 
		//
		//--------------------------------------------------------------------------

		
		c = NULL;
		seek = false;
		
		begin = 0;

		// Open a video file and read header
		int err = av_open_input_file(&ic, filename_, NULL, 0, NULL);
		
		if(err < 0)
		{
			print("Error opening file %s\n", filename_);
			return false;
		}

		// If no header present, try to find infos
		err = av_find_stream_info(ic);
		
		// ptr to the video stream 
		// streams[0]=video and streams[1]=audio
		st = ic->streams[0];

		if(err < 0 ) 
		{
			warning("%s: coulnd not find codec parameters\n", filename_);

			return false;
		}
		
		// Read video parameters
		// usualy, frame rate base = 1
		// nframe could not be read directly, so must compute it
		frate = st->r_frame_rate/st->r_frame_rate_base;
		nframes = (int)(frate * st->duration / AV_TIME_BASE);
		width = st->codec.width;
		height = st->codec.height;

		// pkt = ptr to AVPacket	
		pkt = & packt;

		// test if codec is present or raw video file
		if (st->codec.codec_id == CODEC_ID_NONE) 
		{
			no_codec = true;
		
			readMode = true;

			return true;
		} 
		else 
		{
			codec = avcodec_find_decoder(st->codec.codec_id);
			no_codec = false;
		}

		if(!codec) 
		{
	   		warning("codec not found\n");
			return false;
   	 	}
	
		// just init some values, but no mem allocation
		avframe = avcodec_alloc_frame();

    		// alloc mem for the codec
		c= avcodec_alloc_context();
	
		if(codec->capabilities&CODEC_CAP_TRUNCATED)
    	    	c->flags|= CODEC_FLAG_TRUNCATED; 
  
    		// Open it
		if (avcodec_open(c, codec) < 0) 
		{
     	   		warning("could not open codec\n");

     	   		return false;
    		}
	
		codec_name = (char *) codec->name;
		
    		// Picture allocation for RGB convertion
		pict = &pic1;
        	avpicture_alloc(pict,PIX_FMT_RGB24, width,height);
		pixmap = pict->data[0]; 
		    		
		readMode = true;

		return true;
	}
	else if(strcmp(open_flags_, "w") == 0) 
	{
		//--------------------------------------------------------------------------
		//
		//		W R I T E       I N I T I A L I S A T I O N 
		//
		//--------------------------------------------------------------------------
 
		// Alloc mem for output AVFormatContext oc	
		oc = av_alloc_format_context();
	    
		// Guess file format with file extention
		oc->oformat = guess_format(NULL, filename_, NULL);

		if (!oc->oformat) 
		{
			warning("Unable to find output format for %s\n",filename_);
			
			return false;
		}
	
		// Alloc priv_data for format (to create indexies)
    		oc->priv_data = av_mallocz(oc->oformat->priv_data_size);

		avframe = avcodec_alloc_frame();

		// Create the video stream on output AVFormatContext oc
		st = av_new_stream(oc,0);
		// Alloc the codec to the new stream
		c = &st->codec;
    		
		/* find the video encoder */
    		codec = avcodec_find_encoder(oc->oformat->video_codec);
    		if (!codec) 
		{
        		warning("codec not found\n");

        		return false;
    		}

		codec_name = (char *) codec->name;
		
    	  	// Create the output file
		url_fopen(&oc->pb, filename_, URL_WRONLY);

		// last part of init will be set when first frame write()
		// because we need user parameters like size, bitrate...

		writeMode = true;
		
		return true;
	}

	return false;
#else
	warning("ffmpegVideoFile::open() ffmpeg not supported.");

	return false;
#endif
}

bool ffmpegVideoFile::read()
{
	if(cframe > nframes) 
	{
		warning("End of file.\n");
		
		return false;
	}
		
#ifdef HAVE_FFMPEG	
	bool frame_created=false;
	int len = 0;
	int rot = 0;
	int got_picture = 0;
	unsigned char *ptr=NULL;

	if (no_codec) 
	{
		// If no_codec (RAW rgb file) don't need to decode frame
		// we only read it
		
		if (seek == false) 
		{
			av_init_packet(pkt);
			av_read_frame(ic, pkt);
		}
		rgb24_to_bgr24_flip(pkt->data, NULL, width, height);
		pixmap = pkt->data;
		cframe++;
		
	}
	else 
	{
		while (!frame_created) 
		{
		audio:
			len = 0;
			// set packet values to 0
			//av_init_packet(pkt);
				
			// ------------ SEEKING SUPPORT ------------
			if (seek == true) 
			{
				got_picture = 1;
				len = 0;
				// if seek, don't need to read and decode frame
				// it is already do in seekmovie()
				goto seek;
			}
				
			// read the next frame
			av_read_frame(ic, pkt);
		
			// begin represent the number of non-video/audio frames at begin of file
			// it's important when seeking
			if (cframe==0) begin++;

			len = pkt->size;
	    		ptr = pkt->data;

			// if read frame is audio frame, we ignore it
			if (pkt->stream_index != 0) 
			{
				// added
				//av_free_packet(pkt);
				goto audio;
			}
		
			while (len > 0) 
			{
				// decode the frame and put it into picture
				// rot represent number of read bytes by decoder
	   		     	rot = avcodec_decode_video(c, avframe, &got_picture, ptr, len);

			seek :
				if (rot < 0) 
				{
    			  		warning("Error while decoding frame %d\n", cframe);
					
	   		        	return false;
				}
 
				if (got_picture) 
				{
					//Convert YUV240P(pict_src) to RGB24(pict_dest)
					img_convert(pict,PIX_FMT_RGB24, (AVPicture*)avframe, c->pix_fmt,width, height);
					frame_created=true;
					cframe++;
				}
				len -= rot;
			}
			
			// putting av_free_packet here bugs reading the last frame of the file
			//av_free_packet(pkt);
		}
		
	}
	seek = false;
	return true;
#else
	warning("ffmpegVideoFile::read() ffmpeg not supported.");

	return false;
#endif
}

bool ffmpegVideoFile::write(unsigned char *pixmap_)
{	
#ifdef HAVE_FFMPEG
	int out_size;
	
	// Now we have encoding parameters, we must finish initialisation
	if (cframe == 0) 
	{
		/* ---- codec parameters ---- */
		c->bit_rate = bitrate;
		c->width = width;  
    		c->height = height;
    	   	c->frame_rate = (int)frate;  
    		c->frame_rate_base = 1;
    		c->gop_size = gop; 
		c->me_method = ME_EPZS;
		/* -------------------------- */
		
    		// open it
    		if (avcodec_open(c, codec) < 0) 
		{
       			warning("could not open codec\n");
			
        		return false;
    		}
		
    		// Write header
    		av_write_header(oc);
          
    		// alloc image and output buffer
    		pict = &pic1;
		avpicture_alloc(pict,PIX_FMT_YUV420P, c->width,c->height);

   		outbuf_size = 1000000;
    		outbuf = new uint8_t[outbuf_size];
		avframe->linesize[0]=c->width*3;
	}
	
	avframe->data[0] = pixmap_;
		
	// Convert to YUV240P 
	img_convert(pict,PIX_FMT_YUV420P, (AVPicture*)avframe, PIX_FMT_RGB24,c->width, c->height);
	

        // encode the image
        out_size = avcodec_encode_video(c, outbuf, outbuf_size, (AVFrame*)pict);

	if (av_write_frame(oc, 0, outbuf, out_size)) 
	{
		warning("error while writing frame ...");

		return false;
	}
	cframe++;
	
   	return true;
#else
	warning("ffmpegVideoFile::write() ffmpeg not supported.");

	return false;
#endif
}  

bool ffmpegVideoFile::rewindmovie()
{
#ifdef HAVE_FFMPEG	
	seekmovie(0,0,0,0);

	return true;
#else
	warning("ffmpegVideoFile::rewind() ffmpeg not supported.");

	return false;
#endif
	}

int ffmpegVideoFile::seekmovie(int h, int m, int s, int f)
{
#ifdef HAVE_FFMPEG
	float fFramesLate;
	int nFramesLate,SeekingFrame, got_picture;
	int64_t SeekingPosition;
	bool haveKeyFrame=false;
	int64_t timeLate;
	int len,rot;

	// compute the position to seek
	SeekingFrame = (int)((h*3600 + m*60 + s)*frate + f);
	SeekingPosition = (int64_t)(SeekingFrame/frate*1000000+0.4);
		
	//print("seek to %d:%d:%d and frame %d or %d microseconds\n",h,m,s,f,SeekingPosition);
	
	if (SeekingPosition > ic->duration) 
	{
		print("D'ho! Could not seek after the end...\n");
		return -1;
	}

	if(begin==0) read();
	
	// goto key frame just before
	av_seek_frame(ic, -1, SeekingPosition);
	
	// it normaly works...
	if (no_codec) 
		return SeekingFrame;
 
	// Sometimes, av_seek_frame is late, you need to decode some frames
	// to the next keyframe...
	while (haveKeyFrame == false) 
	{
		av_read_frame(ic, pkt);
  		len = pkt->size;
		if (pkt->stream_index == 0) 
		{
	  		while (len>0) 
			{
				rot = avcodec_decode_video(c, avframe, &got_picture, pkt->data, len);
				if (got_picture && avframe->key_frame) haveKeyFrame = true;
		 		len -= rot;
			}
	  	}
	}

	// Now, we are exactely in keyframe juste before frame wanted	
	//find how many frames between key frame and frame wanted
	
	timeLate =  SeekingPosition - ic->streams[0]->cur_dts;
	fFramesLate = timeLate * frate/ 1000000;
	nFramesLate = (int)fFramesLate;
	
	//decode frame until frame wanted
	for (int i=0;i < nFramesLate+begin;) 
	{
		av_read_frame(ic, pkt);	
		len = pkt->size;
		if (pkt->stream_index == 0) 
		{
			while (len>0) 
			{
				rot = avcodec_decode_video(c, avframe, &got_picture, pkt->data, len);
				len -= rot;
			}
			i++;
		}
	}
	
	// seek support ON
	seek = true;
	return SeekingFrame;

#else
	warning("ffmpegVideoFile::seekmovie() ffmpeg not supported.");

	return -1;
#endif
}

bool ffmpegVideoFile::close()
{
#ifdef HAVE_FFMPEG
	if (readMode) 
	{
		if (!no_codec) avcodec_close(c);
		av_free(c);
		av_close_input_file(ic);
		ic = NULL;
		av_free_packet(pkt);
		
		if (!no_codec) avpicture_free(pict);
		av_free(avframe);
	} 
	else if (writeMode) 
	{
	   	if(cframe > 0)
		{
			// write indexies and update header
			av_write_trailer(oc);
		
			// close output file
			url_fclose(&oc->pb);
			
	 		delete outbuf;
			outbuf = NULL;
    		
			avcodec_close(c);
			
			if (!no_codec) avpicture_free(pict);
			av_free(avframe);
		}
	}
	pixmap = NULL;

	return true;
#else
	warning("ffmpegVideoFile::close() ffmpeg not supported.");

	return false;
#endif
}

ffmpegVideoFile::~ffmpegVideoFile()
{
	 n_instance--;
}

}

