#ifndef RGB_RAW_VIDEO_FILE_INC
#define RGB_RAW_VIDEO_FILE_INC

#include "VideoFile.h"

#ifdef HAVE_AVILIB
extern "C"
{
#include "avilib.h"
}
#endif

namespace Torch {

/** This class is designed to reads/writes raw-RGB AVI files
	
    Warning: RGB images are stored as flipped BGR 24 bits pixmaps.
  
    @author Sebastien Marcel (marcel@idiap.ch) 
    @author Julien Tiphaigne
    @version 2.0
    \Date
    @since 1.0
*/
class rgbRawVideoFile : public VideoFile
{
#ifdef HAVE_AVILIB
   	// Handle for an AVI file in input
	avi_t *avifile_input;

	// Handle for an AVI file in output
	avi_t *avifile_output;
#endif

	//
	bool readMode;

	//
	bool writeMode;

	// size of the current frame (in bytes)
	int size;

public:
	/** @name constructors */
	//@{
	///
  	rgbRawVideoFile();
	
	///
	rgbRawVideoFile(char *filename_, const char *open_flags_ = "r");
	//@}
	
	///
  	virtual ~rgbRawVideoFile();

	/// open the video file #filename#
	virtual bool open(char *filename_, const char *open_flags_ = "r");

	/// read the current image from the file and fill #pixmap#
	virtual bool read();

	/// write #pixmap_# into a file
	virtual bool write(unsigned char *pixmap_);

	/// restart the video file at the beginning
	virtual bool rewindmovie();

	/// seek in the video file
	virtual int seekmovie(int h, int m, int s, int f);
	
	/// close the video file
  	virtual bool close();

};

}

#endif
